using System;
using OpenTK.Graphics.OpenGL4;

namespace Dashboard.OpenGL
{
    [System.Serializable]
    public class GraphicsException : Exception
    {
        public GraphicsException()
        {
            AddExtraData();
        }

        public GraphicsException(string message) : base(message)
        {
            AddExtraData();
        }

        public GraphicsException(string message, Exception inner) : base(message, inner)
        {
            AddExtraData();
        }

        protected GraphicsException(
            System.Runtime.Serialization.SerializationInfo info,
            System.Runtime.Serialization.StreamingContext context) : base(info, context)
        {
            AddExtraData();
        }

        private void AddExtraData()
        {
            GL.GetInteger(GetPName.MajorVersion, out int major);
            GL.GetInteger(GetPName.MinorVersion, out int minor);

            string version = GL.GetString(StringName.Version);
            string vendor   = GL.GetString(StringName.Vendor);
            string renderer = GL.GetString(StringName.Renderer);

            Data.Add("OpenGL Version", new Version(major, minor));
            Data.Add("OpenGL Version String", version);
            Data.Add("OpenGL Vendor", vendor);
            Data.Add("OpenGL Renderer", renderer);
        }
    }
}